"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getExecutionContext = exports.setExecutionContext = exports.getEnvConfig = exports.HandlerFactory = void 0;
const Errors_1 = require("../Errors");
let databaseSecret = undefined;
function HandlerFactory({ oracleDb, secretsManager, router, closeOracleDbConnectionPoolOnExit, createOracleDbConnectionPool, }) {
    const createConnectionPool = closeOracleDbConnectionPoolOnExit !== null && closeOracleDbConnectionPoolOnExit !== void 0 ? closeOracleDbConnectionPoolOnExit : true;
    const closeConnectionPoolOnExit = createOracleDbConnectionPool !== null && createOracleDbConnectionPool !== void 0 ? createOracleDbConnectionPool : false;
    const { getOrCreatePool, closePool } = oracleDb;
    return async (request, context) => {
        context.callbackWaitsForEmptyEventLoop = false;
        if (createConnectionPool) {
            const secretName = getEnvConfig('GC_DATABASE_SECRET_NAME');
            const connectString = getEnvConfig('GC_DATABASE_CONNECT_STRING');
            if (!secretName || !connectString)
                throw new Errors_1.InternalServerError('Required environment variables not found');
            if (!databaseSecret)
                databaseSecret = await secretsManager.getSecret(secretName);
            const { password, user } = databaseSecret;
            await getOrCreatePool({
                connectString,
                password,
                user,
            });
        }
        const executionContext = extractExecutionContext(request, context);
        setExecutionContext(executionContext);
        const { response } = await router.route(request, context);
        if (closeConnectionPoolOnExit)
            await closePool();
        return response;
    };
}
exports.HandlerFactory = HandlerFactory;
function extractExecutionContext(request, context) {
    var _a, _b;
    const identity = getIdentity(request);
    return {
        requestId: context.awsRequestId,
        path: request.path,
        method: request.httpMethod,
        identity: identity,
        sourceIp: (_b = (_a = request.requestContext) === null || _a === void 0 ? void 0 : _a.identity) === null || _b === void 0 ? void 0 : _b.sourceIp,
    };
}
function getIdentity(request) {
    var _a, _b;
    const apiKey = (_b = (_a = request.requestContext) === null || _a === void 0 ? void 0 : _a.identity) === null || _b === void 0 ? void 0 : _b.apiKey;
    if (apiKey) {
        const startOfApiKey = apiKey.substring(0, 5);
        return `apikey ${startOfApiKey}*****`;
    }
    return 'unknown';
}
function getEnvConfig(name) {
    return process.env[name];
}
exports.getEnvConfig = getEnvConfig;
const executionContext = {};
function setExecutionContext(context) {
    executionContext.requestId = context.requestId;
    executionContext.path = context.path;
    executionContext.method = context.method;
    executionContext.identity = context.identity;
    executionContext.sourceIp = context.sourceIp;
}
exports.setExecutionContext = setExecutionContext;
function getExecutionContext() {
    return executionContext;
}
exports.getExecutionContext = getExecutionContext;
