"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateRequest = void 0;
const Errors_1 = require("../Errors");
function validateRequest(schemas) {
    return function validate(_target, _propertyKey, descriptor) {
        let originalMethod = descriptor.value;
        descriptor.value = async function (event, context) {
            const { pathParameters, queryStringParameters, headers, rawBody } = event;
            const { pathParametersSchema, queryStringParametersSchema, headersSchema, bodySchema } = schemas;
            let body;
            if (bodySchema) {
                try {
                    body = typeof rawBody === 'string' && rawBody.trim() ? JSON.parse(rawBody) : {};
                }
                catch (e) {
                    throw new Errors_1.BadRequest('Malformed JSON body');
                }
            }
            try {
                if (pathParametersSchema)
                    await pathParametersSchema.validateAsync(pathParameters);
            }
            catch (e) {
                throw new Errors_1.BadRequest(`In path: ${String(e)}`);
            }
            try {
                if (queryStringParametersSchema)
                    await queryStringParametersSchema.validateAsync(queryStringParameters);
            }
            catch (e) {
                throw new Errors_1.BadRequest(`In querystring: ${String(e)}`);
            }
            try {
                if (headersSchema)
                    await headersSchema.validateAsync(headers);
            }
            catch (e) {
                throw new Errors_1.BadRequest(`In header: ${String(e)}`);
            }
            try {
                if (bodySchema)
                    await bodySchema.validateAsync(body);
            }
            catch (e) {
                throw new Errors_1.BadRequest(`In body: ${String(e)}`);
            }
            return originalMethod.apply(this, [event, context]);
        };
    };
}
exports.validateRequest = validateRequest;
